#ifndef NETEX_H
#define NETEX_H

#include <stdio.h>
#include <ncurses.h>


#define PRINT(msg, args...) printw(msg, ##args); refresh()
#define PRINT_LINE(msg, args...) printw(msg "\n", ##args); refresh()
#define WARN(msg, args...) printw("[Warning]: " msg "\n", ##args); refresh()
#define ERROR(msg, args...) printw("[ERROR]: " msg "\n", ##args); refresh()

#define GET_PERC(num, perc) num * perc / 100

typedef enum socket_ipv
{
    IPv4,
    IPv6,
    Double
} SOCKET_IPV_TYPE;

typedef struct host_info
{
    char* ip;
    struct sockaddr* address;
    size_t addrlen;
    SOCKET_IPV_TYPE ipv_type;
} HOST_INFO;


void netex_init(void);
void netex_shutdown(void);
int setup_socket(int port, const char* hostname, SOCKET_IPV_TYPE ipv_type);
HOST_INFO* resolve_host(const char* hostname, int port, SOCKET_IPV_TYPE ipv_type);
void free_host_info(HOST_INFO* host_info);

#endif //NETEX_H