#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include "strutil.h"
#include "netex.h"

// String will be duplicated (allocated) and need to be freed with the 'strsplfree' function
SPLIT_STR* splstr(const char* str, const char delimiter)
{
    char* dup = strdup(str);
    if (dup == NULL)
    {
        const int error = errno;
        ERROR("Failed to duplicate string, error code: %i", error);
        return NULL;
    }
    const size_t delim_occ = str_token_count(str, delimiter);
    SPLIT_STR* splitted = malloc(sizeof(SPLIT_STR));
    if (splitted == NULL)
    {
        const int error = errno;
        ERROR("Failed to allocate memory, error code: %i", error);
        free(dup);
        return NULL;
    }
    if (delim_occ != 0)
    {
        splitted->count = delim_occ;
        splitted->delimited = malloc(splitted->count * sizeof(char*));
        if (splitted->delimited == NULL)
        {
            const int error = errno;
            ERROR("Allocation failed, error code: %i", error);
            free(splitted);
            free(dup);
            return NULL;
        }
    }
    char delim[2] = "\0";
    delim[0] = delimiter;
    char* token = strtok(dup, delim);
    for (int i = 0; ; i++)
    {
        if (i != 0)
            token = strtok(NULL, delim);
        if (token == NULL)
            break;
        splitted->delimited[i] = token;
    }
    return splitted;
}

void strsplfree(SPLIT_STR* str)
{
    free(*str->delimited); // Free the allocated array and not the ptr pointing to the array!
    free(str);
}

// Returns the amount of times the 'character' occurrs in the given string.
size_t strocc(const char* str, const char character)
{
    size_t occurrences = 0;
    for (int i = 0;; i++)
    {
        if (str[i] == '\0')
            break;
        if (str[i] == character)
            occurrences++;
    }
    return occurrences;
}
// Get the token count if it where to get tokenized (strtok) with the given delimiter.
int str_token_count(const char* str, const char delimiter)
{
    int path_elements = 1;
    for (int i = 0;; i++)
    {
        if (str[i] == '\0')
        {
            if (i == 0)
                break;
            return path_elements;
        }
        if (str[i] == delimiter)
        {
            if (str[i + 1] == delimiter || str[i + 1] == '\0' || i == 0)
                return -1;
            path_elements++;
        }
    }
    return -1;
}